﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NDesk.Options;

namespace uploadlast
{
    class Program
    {
        /* Parametry:
         *  -d = lokalni adresar, ve kterem bude vyhledan soubor s posledni zmenou
         *  -r = URI vzdaleneho serveru vcetne vzdalene slozky
         *  -u = uzivatelske jmeno
         *  -p = heslo
         */
        static void Main(string[] args)
        {
            // Zpracovani parametru.
            AppArguments appArguments = new AppArguments();
            bool showHelp = false;

            var options = new OptionSet()
            {
                {"?|h|help", "Displays this help.", v => showHelp = v != null},
                {"d|dir=", "Local folder. Last modified file will be uploaded.", v => appArguments.LocalFolder = v},
                {"r|remote=", "Remote server URI, including full path. For example: ftp://ftp.server.net/folder1/folder2/.", v => appArguments.RemoteUri = v},
                {"u|user=", "Username.", v => appArguments.User = v},
                {"p|password=", "Password.", v => appArguments.Password = v},
                {"k|keep=", "Number of files to keep on FTP server.", (int v) => appArguments.KeepFiles = v},
                {"t|timestamp", "Add current time to uploaded file name.", v => appArguments.AddTimeStamp = v != null}
            };

            try
            {
                options.Parse(args);
            }
            catch (OptionException e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine();
                ShowHelp(null, options);
            }

            if (showHelp)
                ShowHelp(null, options);

            // Kontrola, zda jsou zadany vsechny povinne parametry.
            if (String.IsNullOrEmpty(appArguments.LocalFolder))
                ShowHelp("Specify local folder.", options);

            if (String.IsNullOrEmpty(appArguments.RemoteUri))
                ShowHelp("Specify remote server URI.", options);

            if (String.IsNullOrEmpty(appArguments.User))
                ShowHelp("Missing username.", options);

            if (String.IsNullOrEmpty(appArguments.Password))
                ShowHelp("Missing password.", options);

            if (appArguments.KeepFiles < 1)
                ShowHelp("`Keep' parameter must be greater than zero.", options);

            // Vyhledani a upload naposledy zmeneneho souboru.
            UploadLast ul = new UploadLast(appArguments);
            ul.UploadLastFile();
        }

        static void ShowHelp(string Message, OptionSet OptionSet)
        {
            if (!String.IsNullOrEmpty(Message))
                Console.WriteLine(Message + Environment.NewLine);

            Console.WriteLine("Usage: " + Environment.NewLine
                + System.AppDomain.CurrentDomain.FriendlyName
                + " -d local folder -r remote server URI -u username -p password [-k integer] [-t]");

            Console.WriteLine();
            Console.WriteLine("Arguments:");
            OptionSet.WriteOptionDescriptions(Console.Out);

            Console.WriteLine(Environment.NewLine
                + "This program contains following components:" + Environment.NewLine
                + "NDesk.Options parser: http://ndesk.org/Options");

            Environment.Exit(-1);
        }
    }
}
