﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Net;

namespace uploadlast
{
    class UploadLast
    {
        private AppArguments appArguments;

        public UploadLast(AppArguments Args)
        {
            appArguments = Args;
        }

        public void UploadLastFile()
        {
            // Kontrola, zda adresar existuje.
            if (!Directory.Exists(appArguments.LocalFolder))
            {
                Console.WriteLine("Can't find folder: " + appArguments.LocalFolder);
                return;
            }

            // Vyhledani naposledy zmeneneho souboru.
            string lastModifiedFile = GetLastModifiedFile(appArguments.LocalFolder);

            if (!String.IsNullOrEmpty(lastModifiedFile))
            {
                Console.WriteLine("Uploading file " + lastModifiedFile);

                try
                {
                    // Upload souboru.
                    using (WebClient ftpClient = new WebClient())
                    {
                        // Bez proxy serveru.
                        ftpClient.Proxy = null;

                        // Prihlasovaci udaje.
                        NetworkCredential credentials = new NetworkCredential(appArguments.User, appArguments.Password);
                        ftpClient.Credentials = credentials;

                        // Slozeni URI (cetsta + lokalni soubor).
                        Uri uri = new Uri(appArguments.RemoteUri);

                        if (appArguments.AddTimeStamp)
                        {
                            string time = DateTime.Now.ToString("-yyyyMMdd-HHmmss", System.Globalization.CultureInfo.InvariantCulture);
                            string fileName = Path.GetFileNameWithoutExtension(lastModifiedFile);
                            string extension = Path.GetExtension(lastModifiedFile);

                            uri = new Uri(uri, fileName + time + extension);
                        }
                        else
                            uri = new Uri(uri, Path.GetFileName(lastModifiedFile));

                        // Upload souboru.
                        byte[] response = ftpClient.UploadFile(uri, lastModifiedFile);

                        // Vypsani odpovedi od FTP serveru.
                        if (response.Length > 0)
                            Console.WriteLine(System.Text.Encoding.ASCII.GetString(response));

                        Console.WriteLine("File uploaded.");
                        Console.WriteLine();

                        // Nacteni souboru ulozenych na FTP serveru.
                        List<RemoteFile> remoteFiles = GetFtpDirectoryListing(credentials);

                        // Ziskani casu posledni zmeny u souboru na FTP serveru.
                        foreach (var remoteFile in remoteFiles)
                        {
                            uri = new Uri(appArguments.RemoteUri);
                            uri = new Uri(uri, remoteFile.Name);
                            
                            remoteFile.LastModified = GetFtpFileModification(credentials, uri);
                        }

                        // Vyhledani starych souboru, ktere budou z FTP serveru smazany.
                        var oldFiles = remoteFiles
                            .Where(f => f.LastModified > DateTime.MinValue)
                            .OrderByDescending(f => f.LastModified)
                            .Skip(appArguments.KeepFiles);

                        foreach (var remoteFile in oldFiles)
                        {
                            Console.WriteLine("Deleting file " + remoteFile.Name);

                            uri = new Uri(appArguments.RemoteUri);
                            uri = new Uri(uri, remoteFile.Name);

                            FtpDeleteFile(credentials, uri);
                        }
                    }
                }
                catch (WebException we)
                {
                    Console.WriteLine(we.Message);

                    if (we.Response.ResponseUri.Scheme == "ftp")
                        Console.WriteLine(((FtpWebResponse)we.Response).StatusDescription);
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.StackTrace);
                    Console.WriteLine();
                    Console.WriteLine(e.Message);
                }
            }
        }

        string GetLastModifiedFile(string Folder)
        {
            string file = String.Empty;

            // Otevreni adresare.
            var di = new DirectoryInfo(Folder);

            try
            {
                // Vyhledani naposledy zmeneneho souboru.
                file = di.GetFiles() // Nacteni vsech souboru.
                    .OrderByDescending(f => f.LastWriteTime) // Serazeni sestupne dle casu zmeny.
                    .Select(f => f.FullName) // Z tridy FileInfo se vrati jen cela jmena souboru.
                    .First(); // Vraceni pouze 1. zaznamu z kolekce.
            }
            catch (InvalidOperationException)
            {
                Console.WriteLine("No file found.");
            }
            catch (Exception e)
            {
                Console.WriteLine(e.StackTrace);
                Console.WriteLine();
                Console.WriteLine(e.Message);
            }

            return file;
        }

        List<RemoteFile> GetFtpDirectoryListing(NetworkCredential credentials)
        {
            List<RemoteFile> remoteFiles = new List<RemoteFile>();

            Uri uri = new Uri(appArguments.RemoteUri);

            var req = (FtpWebRequest)WebRequest.Create(uri);
            req.Proxy = null;
            req.Credentials = credentials;
            req.Method = WebRequestMethods.Ftp.ListDirectory;

            using (var resp = (FtpWebResponse)req.GetResponse())
            {
                using (StreamReader reader = new StreamReader(resp.GetResponseStream()))
                {
                    string line;
                    while ((line = reader.ReadLine()) != null)
                    {
                        //Console.WriteLine(line);
                        remoteFiles.Add(new RemoteFile(line));
                    }
                    //Console.WriteLine(reader.ReadToEnd());
                }

                //Console.WriteLine(resp.ResponseUri.ToString());
            }

            return remoteFiles;
        }

        DateTime GetFtpFileModification(NetworkCredential credentials, Uri uri)
        {
            DateTime lastModified = DateTime.MinValue;

            var req = (FtpWebRequest)WebRequest.Create(uri);
            req.Proxy = null;
            req.Credentials = credentials;
            req.Method = WebRequestMethods.Ftp.GetDateTimestamp;

            using (var resp = (FtpWebResponse)req.GetResponse())
            {
                lastModified = resp.LastModified;
            }

            return lastModified;
        }

        void FtpDeleteFile(NetworkCredential credentials, Uri uri)
        {
            var req = (FtpWebRequest)WebRequest.Create(uri);
            req.Proxy = null;
            req.Credentials = credentials;
            req.Method = WebRequestMethods.Ftp.DeleteFile;

            req.GetResponse().Close();
        }
    }
}
